'use strict';
var _ = require('underscore');
var Immutable = require('immutable');
var Wish = require('Wish');
function World(initialWorld) {
    this.worldState = Immutable.fromJS(initialWorld || {});
    this.watchers = [];
    this._pendingActions = [];
    this.actionsEnabled = true;
    _.bindAll.apply(_, [this].concat(_.functions(this)));
    this._latentPaths = Immutable.Set();
}
;
World.prototype.getIn = function () {
    return this.worldState.getIn.apply(this.worldState, arguments);
};
World.prototype.get = function () {
    return this.worldState.get.apply(this.worldState, arguments);
};
World.prototype.performAction = function performAction(action) {
    if (!action.name)
        throw new Error('Action has no name.');
    if (this.actionsEnabled) {
        this.enqueueAction(action);
    }
};
World.prototype.enqueueAction = function enqueueAction(action) {
    this._pendingActions.push(action);
    if (!this.actionsScheduled()) {
        this.scheduleActions();
    }
};
World.prototype.scheduleActions = function () {
    setTimeout(() => {
        this._reallyPerformActions();
    }, 1);
    this._scheduled = true;
};
World.prototype.actionsScheduled = function actionsScheduled() {
    return this._scheduled;
};
World.prototype._reallyPerformActions = function () {
    this._pendingActions.forEach(this._reallyPerformAction);
    this._pendingActions = [];
    this._scheduled = false;
    this.clearLatentPaths();
    this.watchers.forEach(watcher => {
        return watcher.apply();
    });
};
World.prototype._reallyPerformAction = function (action) {
    if (action.effect)
        action.effect(this);
    if (action.transaction) {
        var newState = action.transaction(this.worldState);
        if (!this.worldState)
            throw new Error('Action\'s transaction returns no worldState');
        this.worldState = newState;
    }
};
World.prototype.onActionPerformed = function onActionPerformed(fn) {
    this.watchers.push(fn);
};
World.prototype.stopPerformingActions = function () {
    this.actionsEnabled = false;
};
World.prototype.clearLatentPaths = function () {
    this._latentPaths = Immutable.Set();
};
World.prototype.findExistingWish = function (path) {
    var wish = this.getIn(path);
    if (!wish) {
        wish = this.findLatentWish(path);
    }
    return wish;
};
World.prototype.findLatentWish = function (path) {
    return this._latentPaths.has(Immutable.List(path)) ? Wish.Pending() : undefined;
};
World.prototype.trackLatentPath = function (path) {
    this._latentPaths = this._latentPaths.add(Immutable.List(path));
};
World.prototype.pathAsPromise = function (path) {
    let getter = () => this.findExistingWish(path);
    return new Promise((resolve, reject) => {
        let wish = getter();
        switch (wish.get('state')) {
        case 'Available':
            resolve(wish.get('value'));
            break;
        case 'Pending':
            this.onActionPerformed(() => {
                wish = getter();
                if (wish.get('state') === 'Available') {
                    resolve(wish.get('value'));
                } else if (wish.get('state') === 'Error') {
                    reject(wish.get('value'));
                }
            });
            break;
        case 'Error':
            reject(wish.get('value'));
            break;
        }
    });
};
module.exports = World;
