var _ = require('underscore');
var {Map, is} = require('immutable');
var {i18n} = require('I18N');
var Validators = {
    allValid: function (valObjs) {
        var ok = v => v.ok;
        var allOk = _.all(_.map(valObjs, ok));
        var errMsgs = _.map(_.reject(valObjs, ok), v => v.errorMsg || '');
        return {
            ok: allOk,
            errorMsg: errMsgs
        };
    },
    gated: function (boolean, validation) {
        return boolean ? validation : _.extend({}, validation, {
            ok: true,
            children: _.mapObject(validation.children, (v, k) => Validators.gated(boolean, v))
        });
    },
    notEmpty: function (string) {
        string = string || '';
        return {
            ok: string !== '',
            errorMsg: i18n('Field must not be empty.')
        };
    },
    emailAddress: function (string) {
        var regex = /^.+@([0-9A-Za-z][0-9A-Za-z.-]+)+\.[A-Za-z]{2,}$/;
        return {
            ok: regex.test(string),
            errorMsg: i18n('Please enter an email address.')
        };
    },
    lengthInRange: function (string, smallest, biggest) {
        var stringLength = (string || '').length;
        return {
            ok: stringLength >= smallest && stringLength <= biggest,
            errorMsg: 'Field must be between ' + smallest + ' and ' + biggest + ' characters.'
        };
    },
    longerThan: function (string, tooShort) {
        string = string || '';
        return {
            ok: string.length > tooShort,
            errorMsg: 'Field must be at least ' + (tooShort + 1) + ' characters long.'
        };
    },
    shorterThan: function (string, tooLong) {
        string = string || '';
        return {
            ok: string.length < tooLong,
            errorMsg: 'Field must be fewer than ' + tooLong + ' characters.'
        };
    },
    location: function (location) {
        function isNotApplicable(x) {
            return is(x, Map({ Geo: 'NotApplicable' }));
        }
        var isAutocompleted = function () {
            return !_.isEmpty(location.get('location')) && _.isNumber(location.get('geoid'));
        };
        if (location.get('fallback')) {
            var children = {
                city: {
                    ok: !_.isEmpty(location.get('city')),
                    errorMsg: i18n('City must not be blank.')
                },
                country: {
                    ok: _.isNumber(location.get('countryGeoid')),
                    errorMsg: i18n('Please choose a country.')
                },
                region: {
                    ok: _.isNumber(location.get('regionGeoid')) || isNotApplicable(location.get('regionGeoid')),
                    errorMsg: i18n('Please choose a region.')
                }
            };
            var ok = _.every(children, c => c.ok);
            return {
                ok: ok,
                errorMsg: i18n('Please enter a valid location.'),
                children: children
            };
        } else {
            return {
                ok: isAutocompleted(),
                errorMsg: i18n('Please select a location')
            };
        }
    },
    passedCaptcha: function (captcha) {
        return {
            ok: !_.isEmpty(captcha.get('response')),
            errorMsg: i18n('Captcha must be completed.')
        };
    }
};
module.exports = Validators;
