'use strict';
var _ = require('underscore');
var cookie = require('cookie');
var Immutable = require('immutable');
var API = require('API');
var Router = require('Router');
var Wish = require('Wish');
var LegacyI3APIHeaders = require('legacy/I3APIHeaders');
function AttemptToLogIn(path, credentials) {
    var route;
    var type = 'json';
    if (credentials.login && credentials.password) {
        route = Router.route('api/v2/sessions/create');
    } else if (credentials.janrainToken) {
        route = Router.route('engage-token');
        credentials = { token: credentials.janrainToken };
        type = 'form';
    } else {
        throw new Error('Invalid credentials object:', credentials);
    }
    return {
        name: 'Auth/AttemptToLogIn',
        transaction: ws => ws.setIn(path, Wish.Pending()),
        effect: world => API.request(world, route, _.partial(LoginAttemptSucceeded, path), _.partial(LoginAttemptFailed, path), {
            payload: credentials,
            method: 'POST',
            type: type,
            extraHeaders: credentials.token ? LegacyI3APIHeaders : {}
        })
    };
}
function LoginAttemptSucceeded(path, value) {
    return {
        name: 'Auth/LoginAttemptSucceeded',
        transaction: ws => {
            function wanted(value, key) {
                return !Immutable.Set([
                    'componentInfo',
                    'redirect',
                    'systemMessages',
                    'updates'
                ]).has(key);
            }
            var userInfo = value.filter(wanted);
            var newWorld = ws.setIn(path, Wish.Available(userInfo));
            if (userInfo.get('primary_key') || userInfo.get('primaryKey')) {
                newWorld = newWorld.setIn([
                    'App',
                    'User'
                ], Wish.Available(userInfo)).set('Actions', Immutable.Map());
            }
            return newWorld;
        }
    };
}
function LoginAttemptFailed(path, error) {
    return {
        name: 'Auth/LoginAttemptFailed',
        transaction: ws => ws.setIn(path, error.get('status').toString().match(/^4/) ? Wish.Available(error.set('error', true)) : Wish.Error(error))
    };
}
function AttemptToSignUpByEmail(path, payload) {
    var route = Router.route('api/v2/users');
    return {
        name: 'Auth/AttemptToSignUpByEmail',
        transaction: ws => ws.setIn(path, Wish.Pending()),
        effect: world => API.request(world, route, _.partial(SignupAttemptSucceeded, path), _.partial(SignupAttemptFailed, path), {
            method: 'POST',
            payload: payload
        })
    };
}
function SignupAttemptSucceeded(path, mapper, value) {
    return {
        name: 'Auth/SignupAttemptSucceeded',
        transaction: ws => ws.setIn(path, Wish.Available(true)).setIn([
            'App',
            'User'
        ], undefined)
    };
}
function SignupAttemptFailed(path, error) {
    return {
        name: 'Auth/SignupAttemptFailed',
        transaction: ws => ws.setIn(path, error.get('status').toString().match(/^4/) ? Wish.Available(error.set('error', true)) : Wish.Error(error))
    };
}
function RequestPasswordResetForEmail(path, email) {
    var payload = {
        'content_prompt_email': email,
        'content_current': 'prompt',
        'content_goto_forward': 1
    };
    var route = Router.route('forgot-password');
    return {
        name: 'Auth/RequestPasswordResetForEmail',
        transaction: ws => ws.setIn(path, Wish.Pending()),
        effect: world => API.request(world, route, _.partial(PasswordResetRequestSucceeded, path), _.partial(PasswordResetRequestFailed, path), {
            method: 'POST',
            type: 'form',
            payload: payload,
            extraHeaders: LegacyI3APIHeaders
        })
    };
}
function PasswordResetRequestSucceeded(path, mapper, response) {
    return {
        name: 'Auth/PasswordResetRequestSucceeded',
        transaction: ws => ws.setIn(path, Wish.Available(true))
    };
}
function PasswordResetRequestFailed(path, error) {
    return {
        name: 'Auth/PasswordResetRequestFailed',
        transaction: ws => ws.setIn(path, Wish.Available(false))
    };
}
var Auth = {
    userExistsWithEmail(world, email) {
        return API.get(world, [
            'Auth',
            'userExistsWithEmail',
            email
        ], Router.route('api/v2/user/check', { email: email }), result => result.get('exists'));
    },
    janrainLoginAttemptPath(token) {
        return [
            'Auth',
            'janrainLoginAttempt',
            token
        ];
    },
    emailLoginAttemptPath(login, password) {
        return [
            'Auth',
            'emailLoginAttempt',
            login,
            password
        ];
    },
    logInByJanrain(world, janrainToken) {
        var path = this.janrainLoginAttemptPath(janrainToken);
        return Wish.performOnce(world, path, AttemptToLogIn(path, { janrainToken }));
    },
    logInByEmail(world, login, password) {
        var path = this.emailLoginAttemptPath(login, password);
        world.performAction(AttemptToLogIn(path, {
            login,
            password
        }));
    },
    loginAttemptWithEmailAndPassword(world, login, password) {
        return world.getIn(this.emailLoginAttemptPath(login, password));
    },
    loginAttemptWithJanrainToken(world, janrainToken) {
        return world.getIn(this.janrainLoginAttemptPath(janrainToken));
    },
    existingResetRequestForEmail(world, email) {
        return world.getIn([
            'Auth',
            'passwordResetRequest',
            email
        ]);
    },
    requestPasswordResetForEmail(world, email) {
        var path = [
            'Auth',
            'passwordResetRequest',
            email
        ];
        return Wish.performOnce(world, path, RequestPasswordResetForEmail(path, email));
    },
    signupAttemptPath(payload) {
        return [
            'Auth',
            'signupAttempt',
            payload
        ];
    },
    signupAttemptWithPayload(world, payload) {
        return world.getIn(this.signupAttemptPath(payload));
    },
    signUpByEmail(world, payload) {
        return world.performAction(AttemptToSignUpByEmail(this.signupAttemptPath(payload), payload));
    },
    emailAddressPendingVerification(world) {
        var cookies = typeof Document === 'undefined' ? world.getIn([
            'ServerOnly',
            'Request',
            'cookies'
        ]).toJS() : cookie.parse(document.cookie);
        if (cookies.i3_pending_verification) {
            return cookies.i3_pending_verification;
        }
    },
    verificationDismissed(world) {
        return !!world.getIn([
            'SystemMessages',
            'emailDismissed'
        ]);
    },
    isDefinitelyAuthed(world) {
        return world.getIn([
            'App',
            'User',
            'state'
        ]) === 'Available' && !this.isUnauthed(world);
    },
    isDefinitelyUnauthed(world) {
        return world.getIn([
            'App',
            'User',
            'state'
        ]) === 'Available' && this.isUnauthed(world);
    },
    isUnauthed(world) {
        return !world.getIn([
            'App',
            'User',
            'value',
            'primaryKey'
        ]) && !world.getIn([
            'App',
            'User',
            'value',
            'primary_key'
        ]);
    }
};
module.exports = Auth;
