'use strict';
var _ = require('underscore');
var superagent = require('superagent');
var Immutable = require('immutable');
var FormController = require('FormController');
var Router = require('Router');
var API = require('API');
var Analytics = require('dealers/AnalyticsDealer');
var Org = require('schemas/Org');
let state;
class OrgIntakeForm extends FormController {
    initForm(rootComponent) {
        state = Org.fromDefaults();
        this.root = rootComponent;
    }
    get state() {
        return state;
    }
    getMember(path) {
        let root = state;
        if (typeof path === 'string') {
            return root.members[path];
        } else {
            return path.reduce((el, p) => el.members[p], root);
        }
    }
    _set(path, val) {
        val = val && val.toJS ? val.toJS() : val;
        this.getMember(path).set(val);
    }
    setField(fieldOrPath, val) {
        this._set(fieldOrPath, val);
        this.forceUpdateRoot();
    }
    setMany(pairs) {
        pairs.forEach((value, key) => this._set(key, value));
        this.forceUpdateRoot();
    }
    getField(fieldOrPath) {
        return Immutable.fromJS(this.getMember(fieldOrPath).value);
    }
    getStateAsPayload() {
        var type = this.getField('type');
        if (!type) {
            throw new Error('JSON-API spec requires a type.');
        }
        return {
            data: {
                type: type,
                attributes: state.value
            }
        };
    }
    emptyDeduperResults(world) {
        this._setDedupeResults(world, Immutable.List());
    }
    getAddress() {
        var address = this.getField('addresses').get(0);
        return address || Immutable.Map({});
    }
    dedupeResultsFor(world, term) {
        if (term.length <= 2) {
            this.emptyDeduperResults(world);
            return;
        }
        var url = Router.urlForRoute(Router.route('_data/org-intake-lookup'));
        var location = this.getAddress().get('location');
        if (!location) {
            location = this.getAddress().get('regionName') + ', ' + this.getAddress().get('countryName');
        }
        superagent.get(url).timeout(10000).query({
            q: term,
            geoid: this.getAddress().get('geoid'),
            location_string: location,
            type: this.getField('type')
        }).end(function (res) {
            var dedupes = Immutable.List();
            if (res.status === 200) {
                dedupes = Immutable.fromJS(res.body);
            }
            this._setDedupeResults(world, dedupes);
        }.bind(this));
    }
    createNewOrgEntryForTerm(term) {
        return {
            isAddNew: true,
            name: term
        };
    }
    _setDedupeResults(world, results) {
        world.performAction({
            name: 'OrgIntakeForm/dedupeResults',
            transaction: ws => ws.setIn([
                'OrgIntake',
                'dedupeResults'
            ], results)
        });
    }
    getDedupeResults(world, term) {
        let results = world.getIn([
            'OrgIntake',
            'dedupeResults'
        ]);
        if (results && !results.find(res => res.get('isExact'))) {
            results = results.unshift(Immutable.fromJS(this.createNewOrgEntryForTerm(term)));
        }
        return results;
    }
    selectOrg(orgObject) {
        this._selectedOrg = orgObject;
        this.forceUpdateRoot();
    }
    selectCurrentName() {
        this.selectOrg(this.createNewOrgEntryForTerm(this.state.members.fullName.value));
    }
    selectedOrgItem() {
        return this._selectedOrg;
    }
    existingOrgIsSelected() {
        return typeof this._selectedOrg !== 'undefined' && !!this._selectedOrg.url;
    }
    existingOrgWasReset(world) {
        this.emptyDeduperResults(world);
        this._selectedOrg = undefined;
        this._orgIsResetting = true;
        this.forceUpdateRoot();
    }
    v2WireRepresentation(state) {
        state = Immutable.fromJS(state.value);
        var rep = _.pick(state.toJS(), 'ein', 'keywords', 'website');
        var addr = this.getAddress().toJS();
        if (_.isString(addr.regionCode)) {
            addr.regionCode = addr.regionCode.replace('.', '_');
        }
        return Object.assign(rep, {
            'org_type': Org.CAPITALIZED_TYPES[state.get('type')],
            'short_name': state.get('shortName'),
            'name': state.get('fullName'),
            'mission': state.get('description'),
            'address_geo_autocomplete_location': addr.location,
            'address_geo_autocomplete_geoid': addr.geoid,
            'address_geo_mode': addr.fallback ? 'fallback' : 'autocomplete',
            'address_geo_fallback_city': addr.city,
            'address_geo_fallback_state': addr.regionCode,
            'address_geo_fallback_country': addr.countryCode,
            'address_country_name': addr.countryName,
            'address_state_name': addr.regionName,
            'address_street_address': state.get('streetAddress'),
            'address_delivery_details': state.get('deliveryDetails'),
            'logo': state.get('imageUrl'),
            'activation_actions': state.get('activationActions')
        });
    }
    v3SubmitOrg(world) {
        var orgType = state.get('type');
        var route = Router.route('api/v3/' + orgType + 's/create');
        var org = this.getStateAsPayload();
        this._submitOrg(world, route, org);
    }
    v2SubmitOrg(world) {
        var route = Router.route('api/v2/orgs/create');
        var org = this.v2WireRepresentation(state);
        this._submitOrg(world, route, org);
    }
    get submittingPath() {
        return [
            'OrgIntake',
            'submitting'
        ];
    }
    isSubmitting(world) {
        return world.getIn(this.submittingPath);
    }
    _submitOrg(world, route, org) {
        world.performAction({
            name: 'SubmitOrgIntake',
            transaction: ws => ws.setIn(this.submittingPath, true)
        });
        API.request(world, route, (data, headers) => {
            return {
                name: 'submitOrg/success',
                transaction: ws => ws,
                effect: () => {
                    Analytics.trackPageview('/virtual/add-org/complete', () => {
                        document.location.href = headers.get('location');
                    });
                }
            };
        }, error => {
            return {
                name: 'submitOrg/error',
                transaction: ws => ws.setIn(this.submittingPath, false),
                effect: () => {
                    let fieldErrors = error.getIn([
                        'body',
                        'fieldErrors'
                    ]);
                    fieldErrors && fieldErrors.forEach((errors, field) => {
                        field = { 'name': 'fullName' }[field] || field;
                        errors.forEach(e => state.members[field].addError(e));
                    });
                    this.forceUpdateRoot();
                }
            };
        }, {
            method: 'POST',
            payload: org
        });
    }
}
global.__form__ = module.exports = new OrgIntakeForm();
